#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "webutil.h"
#include "food.h"

/*----------------------------------------------------------------------------*/

typedef struct
{
   char *name;
   char *val;
} entry;

/*----------------------------------------------------------------------------*/

static char* region_category[] = {
   "Downtown",
   "Uptown",
   "East",
   "West",
   "North",
   0
};

static long int region_mask[] = {
   2047,
   3072,
   1015808,
   66060288,
   2080374784,
   0
};

static char* price_category[] = {
   "$30-",
   "$30-$60",
   "$60-$100",
   "$100+",
   "Unknown",
   0
};

static char* cuisine_category[] = {
   "African",
   "Bar/Club/Pub",
   "Cafe/Bistro",
   "Canadian/American",
   "Caribbean",
   "Chinese",
   "Deli/Diner",
   "East-European",
   "French",
   "Greek",
   "Indian",
   "Italian",
   "Japanese",
   "Latin/Spanish/Portugese",
   "Middle-Eastern",
   "Seafood",
   "South-Asian",
   "Steak House",
   "Tex/Mex",
   "Vegetarian",
   "Wine Bar",
   "Other",
   0
};

static char *location_category[] = {
   "Annex",
   "Cabbagetown",
   "Central",
   "Chinatown/Kensington",
   "Esplanade",
   "Financial District",
   "Harbourfront",
   "Queen St. W.",
   "Theatre District",
   "Yorkville",
   "Mount Pleasant",
   "Rosedale",
   "Yonge & Eglinton",
   "Yonge & St. Clair",
   "Beaches",
   "Danforth",
   "East End",
   "Gerrard East",
   "Scarborough",
   "St. Clair West",
   "Eglinton West",
   "Etobicoke",
   "High Park",
   "Little Italy",
   "Mississauga",
   "Markham",
   "North York",
   "Richmond Hill",
   "Thornhill",
   "Woodbridge/Brampton",
   0
};

/*----------------------------------------------------------------------------*/

#define STAT_FMT "%-25s %4d records, %4.1f%%%c"

static void Category( char* title, char* s[], int n[], int ncat, int nitems )
{
   int i;
   printf( "<H3>Breakdown by %s:</H3>%c<PRE>%c", title, 10, 10 );
   for( i = 0; i < ncat; i++ )
   {
      printf( STAT_FMT, s[i], n[i],
      100.0 * (float) n[i] / (float) nitems, 10 );
   }
   printf( "</PRE>%c", 10 );
}

/*----------------------------------------------------------------------------*/

static void DumpStats( FILE* fp )
{
   char buffer[512];
   char name[128], path[256], addr[128], landmark[128], city[128], phone[128];

   int location, locations[30];
   int price, prices[5];
   long int type; int types[22];
   int regions[5];

   int i, n = 0;

   for( i = 0; i <  5; i++ ) prices[i] = 0;
   for( i = 0; i < 30; i++ ) locations[i] = 0;
   for( i = 0; i <  5; i++ ) regions[i] = 0;
   for( i = 0; i < 22; i++ ) types[i] = 0;

   while( fgets( buffer, 512, fp ) != 0 )
   {
      sscanf( buffer, DB_FMT, name, path, addr, landmark, &location, city,
              phone, &type, &price );

      prices[price-1]++;
      locations[location-1]++;
      for( i = 1; i < 23; i++ )
         if( type & pow2( i ) )
            types[i-1]++;
      for( i = 0; i < 5; i++ )
         if( pow2( location ) & region_mask[i] )
            regions[i]++;
      n++;
   }
   printf( "<H3>Total number of records:</H3>%c", 10 );
   printf( "<PRE>%d</PRE>%c", n, 10 );

   Category( "price", price_category, prices, 5, n );
   Category( "location", location_category, locations, 30, n );
   Category( "region", region_category, regions, 5, n );
   Category( "cuisine type", cuisine_category, types, 22, n );

   printf( SIG, ROOT_URL, ROOT_URL, ROOT_URL, 10 );
}

/*----------------------------------------------------------------------------*/

int main( int argc, char* argv[] )
{
   char date[80];
   FILE *fp;

   printf( "Content-type: text/html%c%c", 10, 10 );

   if( strcmp( getenv( "REQUEST_METHOD" ), "POST" ) )
   {
      printf( "This script should be referenced with a METHOD of POST.\n" );
      printf( "If you don't understand this, see this ");
      printf( "<A HREF=\"http://www.ncsa.uiuc.edu/SDG/Software/Mosaic/Docs/fill-out-forms/overview.html\">forms overview</A>.%c", 10 );
      exit( 1 );
   }
   if( strcmp( getenv("CONTENT_TYPE"), "application/x-www-form-urlencoded") )
   {
      printf( "This script can only be used to decode form results.\n" );
      exit( 1 );
   }

   fp = popen( "date -u", "r" );
   fgets( date, 80, fp );
   pclose( fp );

   printf( "<TITLE>Toronto's WWW Restaurant Listings -- Statistics</TITLE>%c",
   10 );
   printf( "<H2>On-Demand Database Statistics</H2>%c", 10 );
   printf( "<P>Compiled on <I>%s</I>%c<HR>%c", date, 10, 10 );

   fp = fopen( DB, "r" );
   DumpStats( fp );
   fclose( fp );

   return 0;
}

/*----------------------------------------------------------------------------*/
